// Copyright 2023, UChicago Argonne, LLC
// All Rights Reserved
// Software Name: NEML2 -- the New Engineering material Model Library, version 2
// By: Argonne National Laboratory
// OPEN SOURCE LICENSE (MIT)
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#include <catch2/catch.hpp>

#include "utils.h"
#include "neml2/tensors/user_tensors/LinspaceFixedDimTensor.h"
#include "neml2/tensors/tensors.h"

using namespace neml2;

#define test_LinspaceFixedDimTensor(tensor_type, tensor_name, batch_shape, nstep, dim)             \
  SECTION("Linspace" #tensor_type)                                                                 \
  {                                                                                                \
    const auto & tensor_name = Factory::get_object<tensor_type>("Tensors", #tensor_name);          \
    const auto & tensor_name##_start =                                                             \
        Factory::get_object<tensor_type>("Tensors", #tensor_name "0");                             \
    const auto & tensor_name##_end =                                                               \
        Factory::get_object<tensor_type>("Tensors", #tensor_name "1");                             \
    const auto tensor_name##_correct =                                                             \
        tensor_type::linspace(tensor_name##_start, tensor_name##_end, nstep, dim);                 \
    REQUIRE(tensor_name.batch_sizes() == batch_shape);                                             \
    REQUIRE(tensor_name.base_sizes() == tensor_type::const_base_sizes);                            \
    REQUIRE(torch::allclose(tensor_name, tensor_name##_correct));                                  \
  }                                                                                                \
  static_assert(true)

TEST_CASE("LinspaceFixedDimTensor", "[tensors/user_tensors]")
{
  load_model("unit/tensors/user_tensors/test_LinspaceFixedDimTensor.i");

  TorchShape B{100, 2, 1};
  TorchSize nstep = 100;
  TorchSize dim = 0;

  test_LinspaceFixedDimTensor(Scalar, a, B, nstep, dim);
  test_LinspaceFixedDimTensor(Vec, b, B, nstep, dim);
  test_LinspaceFixedDimTensor(Rot, c, B, nstep, dim);
  test_LinspaceFixedDimTensor(R2, d, B, nstep, dim);
  test_LinspaceFixedDimTensor(SR2, e, B, nstep, dim);
  test_LinspaceFixedDimTensor(R3, f, B, nstep, dim);
  test_LinspaceFixedDimTensor(SFR3, g, B, nstep, dim);
  test_LinspaceFixedDimTensor(R4, h, B, nstep, dim);
  test_LinspaceFixedDimTensor(SSR4, i, B, nstep, dim);
  test_LinspaceFixedDimTensor(R5, j, B, nstep, dim);
  test_LinspaceFixedDimTensor(SSFR5, k, B, nstep, dim);
}
